<?php


namespace FiloBlu\ExtInventory\Plugin\Magento\Sales\Api;

use FiloBlu\ExtInventory\Model\ShippingAlgorithms\SourceSlots\Algorithm;
use FiloBlu\ExtInventory\Api\InventoryFbOrderitemSourceRepositoryInterface;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterface;
use FiloBlu\ExtInventory\Model\ShippingAlgorithms\SourceSlots\SourcesData;

class OrderManagementInterface
{

    public $algorithm;
    public $InventoryFbOrderitemSourceInterface;
    public $InventoryFbOrderitemSourceRepositoryInterface;
    public $sourcesListByCode;
    public $SourcesData;

    public function __construct(
        Algorithm $algorithm,
        InventoryFbOrderitemSourceRepositoryInterface $InventoryFbOrderitemSourceRepositoryInterface,
        InventoryFbOrderitemSourceInterface $InventoryFbOrderitemSourceInterface,
        SourcesData $SourcesData
    )
    {
        $this->algorithm = $algorithm;
        $this->InventoryFbOrderitemSourceRepositoryInterface = $InventoryFbOrderitemSourceRepositoryInterface;
        $this->InventoryFbOrderitemSourceInterface = $InventoryFbOrderitemSourceInterface;
        $this->SourcesData = $SourcesData;
    }

    /*
     * TODO: Dynamic stockId
     */
    public function afterPlace(
        \Magento\Sales\Api\OrderManagementInterface $subject,
        $order
    ) {

        try {
            $stockId = 2;
            $algorithmPreview = $this->algorithm->getResultPreviewFromOrder($order,$stockId);
            $this->sourcesListByCode = array_flip($this->SourcesData->getSourcesArrayById(2));

            // Algorithm returned zero results
            if (count($algorithmPreview) == 0){
                return $order;
            }

            $bestSolution = $algorithmPreview[0];
            unset($bestSolution['_points']); //Removing debug data

            // Finding how many sources are involved
            $sources_involved = array();
            foreach ($bestSolution as $source_code => $items) {
                foreach ($items as $sku => $qty) {
                    if($qty > 0){
                        $sources_involved[] = $source_code;
                    }
                }
            }
            $sources_involved = array_values(array_unique($sources_involved));

            if(count($sources_involved) == 1){
                $final_source_id = $this->sourcesListByCode[$sources_involved[0]];
            } else {
                $final_source_id = $this->sourcesListByCode[$this->SourcesData->getMainSourceCode()];
            }

            foreach ($bestSolution as $source_code => $items) {

                if (!isset($this->sourcesListByCode[$source_code])){
                    // source_code not found, do what? throw error?
                }
                // Adding data on the support table (only if this is a 'mulstistock' order?)
                foreach ($items as $sku => $qty) {
                    if ($qty < 1){continue;}
                    $orderItem = $this->findOrderItemFromSku($order,$sku);
                    $InventoryFbOrderitemSourceInterface = $this->InventoryFbOrderitemSourceInterface;
                    $InventoryFbOrderitemSourceInterface->setOrderId($order->getEntityId())
                        ->setOrderItemId($orderItem->getItemId())
                        ->setInventorySourceId($this->sourcesListByCode[$source_code])
                        ->setFinalInventorySourceId($final_source_id)
                        ->setSku($sku)
                        ->setQty($orderItem->getQtyOrdered());
                    $this->InventoryFbOrderitemSourceRepositoryInterface->save($InventoryFbOrderitemSourceInterface);
                }
            }
        } catch (\Exception $e) {
            // TODO: Throw exception and prevent order from beign closed?
        }

        return $order;
    }

    public function findOrderItemFromSku($order,$simple_sku){
        $items = $order->getAllItems();
        foreach ($items as $item) {
            if(!$item->getParentItem() && $item->getSku() == $simple_sku){
                return $item;
            }
        }
        return false;
    }
}